///
/// @file  GUI.hpp
/// @param Project: TestSine
/// @author Erik Zorzin
/// @date 22OCT2015
/// @version 1.0.0
///

#ifndef GUI_hpp
#define GUI_hpp

#include "LinkDef.hpp"
#include <TROOT.h>
#include <TApplication.h>
#include <TVirtualX.h>
#include <TQObject.h>
#include <TGWindow.h>
#include <TGSlider.h>
#include <TGClient.h>
#include <TCanvas.h>
#include <TGraph2D.h>
#include <TAxis.h>
#include <TFrame.h>
#include <TPad.h>
#include <TRootEmbeddedCanvas.h>
#include <TGFrame.h>
#include <TPad.h>
#include <TMath.h>
#include <TSystem.h>
#include <TTimer.h>
#include <TStopwatch.h>

const unsigned int  N = 20;                                                                                                     // Number of x-points in the graph [#].
const unsigned int  M = 20;                                                                                                     // Number of y-points in the graph [#].
const double        FS = 20;                                                                                                    // Sampling frequency [Hz].
const double        F_x = 1;                                                                                                    // "x" signal frequency [Hz].
const double        F_y = 1.5;                                                                                                  // "y" signal frequency [Hz].
const double        PI = TMath::Pi();                                                                                           // PI = 3.141....
const double        PHI = 2*PI/360*5;                                                                                           // Phase shift [rad].

class TGWindow;
class TGMainFrame;
class TRootEmbeddedCanvas;


/// @class GUI
/// @brief The GUI - Graphics User Interface class.
/// @discussion This class implements a graphics user interface.
/// @date 22OCT2015
/// @author Erik Zorzin
/// @version 1.0.0
class GUI                                                                                                                       // GUI: Graphic User Interface.
{
private:
    TGMainFrame*                fMain;                                                                                          // Main frame.
    TRootEmbeddedCanvas*        fEcanvas;                                                                                       // Embedded Canvas.
    TCanvas*                    fCanvas;                                                                                        // Canvas.
    TGHorizontalFrame*          hframe;                                                                                         // Horizontal frame.
    TGHSlider*                  A_sld;                                                                                          // Amplitude slider [m].
    TGraph2D*                   graph;                                                                                          // Graph.
    TPad*                       pad;                                                                                            // Pad.
    double                      A = 1;                                                                                          // Signal amplitude [m].
    int                         i;                                                                                              // Integer counter.
    int                         j;                                                                                              // Integer counter.
    int                         k;                                                                                              // Integer counter.
    double                      omega_x = 2*PI*F_x;                                                                             // "omega_x" pulsation [rad/s].
    double                      omega_y = 2*PI*F_y;                                                                             // "omega_y" pulsation [rad/s].
    double                      ds = 1.0/FS;                                                                                    // "dt" [s].
    double                      x;                                                                                              // "x" variable: displacement [m].
    double                      y;                                                                                              // "y" variable: displacement [m].
    double                      z;                                                                                              // "z" variable: displacement [m].
    double                      phi_x = 0;                                                                                      // "phi_x" variable: phase shift [rad].
    double                      phi_y = 0;                                                                                      // "phi_y" variable: phase shift [rad].
    TStopwatch*                 watch;
    
public:
    GUI(const TGWindow* p, UInt_t w, UInt_t h);
    virtual ~GUI();
    
    void DoSlider(Int_t pos);                                                                                                   ///< Manages "Amplitude" [m] slider.
    void CloseApplication();                                                                                                    ///< Manages window's close event.
    void Plot();
};

#endif /* GUI_hpp */
